@@ -105,6 +105,7 @@
 #include INC_XAW(SmeLine.h)
 #include INC_X11(IntrinsicP.h)
 #include INC_XAW(TextP.h)
+#include INC_XAW(Scrollbar.h)
 #include INC_XMU(StdCmap.h)
 #include "Aaa.h"
 #include "Button.h"
@@ -303,41 +304,43 @@ misc_setPageMarker(entry,kind,event,check_toc)
   XEvent *event;
   Boolean check_toc;
 {
-  int x,y,yl,yu,ny=99999;
+  int firstvisible, lastvisible;
   Boolean b = False;
   INFMESSAGE(misc_setPageMarker)
   if (toc_text && (entry >= 0)) {
-    if (kind == 0 && VlistSelected(newtoc) != entry)  
+    if (kind == 0 && VlistSelected(newtoc) != entry)
       VlistChangeSelected(newtoc,entry,XawVlistSet);
-    else if (kind == 1 && VlistHighlighted(newtoc) != entry)  
+    else if (kind == 1 && VlistHighlighted(newtoc) != entry)
       VlistChangeHighlighted(newtoc,entry,XawVlistSet);
     else if (kind == 2) {
       entry=VlistSelected(newtoc);
       if (entry<0) return;
     }
-    VlistPositionOfEntry(newtoc,entry,&yu,&yl);
-    x = newtocControl->core.x;
-    y = newtocControl->core.y;
-    IIMESSAGE(x,y)
-    IIMESSAGE(yu,yl)
-    IIMESSAGE(y,newtoc->core.y)
-    if (yu != yl) {
-      ny = -((int)newtoc->core.y + yu) + 14;
-      if (y<0 && y < ny) b = True;
-      if (!b) {
-	ny = (int)newtocClip->core.height - ((int)newtoc->core.y + yl + 14);
-        if (y>ny) b = True;
+    firstvisible = VlistGetFirstVisible(newtoc);
+    if (firstvisible > entry || (entry > 0 && firstvisible >= entry)) {
+      if (entry > 0)
+	VlistSetFirstVisible(newtoc, entry - 1);
+      else
+	VlistSetFirstVisible(newtoc, entry);
+      b = True;
+    } else {
+      /* sadly newtoc does not know it's height, so it cannot be told
+       * to made an item visible and we need to trick: */
+      lastvisible = VlistEntryOfPosition(newtoc, newtocClip->core.height);
+      if (entry > firstvisible && entry >= lastvisible) {
+	VlistSetFirstVisible(newtoc, entry - (lastvisible - firstvisible - 1));
+	b = True;
       }
-      if (b) {
-	INFIMESSAGE(jumping to,ny)
-	ClipWidgetSetCoordinates(newtocClip,x,ny);
-	if (event)
+    }
+    /* if this happened by mouse, we have the information to get the
+     * highlight up to date again: */
+    if (b) {
+      if (event)
 	{
-	   entry = VlistEntryOfPosition(newtoc,(int)event->xbutton.y+(y-ny));
-	   if (entry != VlistHighlighted(newtoc) && check_toc)
-		VlistChangeHighlighted(newtoc,entry,XawVlistSet);
-        }
-      }
+	  entry = VlistEntryOfPosition(newtoc, (int)event->xbutton.y);
+	  if (entry != VlistHighlighted(newtoc) && check_toc)
+	    VlistChangeHighlighted(newtoc,entry,XawVlistSet);
+	}
     }
   }
 }
@@ -1050,6 +1053,9 @@ setup_ghostview()
       }
       XtSetValues(newtoc, args, n);
       ClipWidgetSetCoordinates(newtocClip,0,0);
+      XawScrollbarSetThumb(newtocScroll,
+		      VlistScrollPosition(newtoc),
+		      VlistVisibleLength(newtoc,newtocClip->core.height));
       GV_XtFree(s);
     }
 
@@ -1064,7 +1070,7 @@ setup_ghostview()
     misc_setSensitive(w_printAllPages     , show_printAllPages     , (gv_psfile   != NULL));
     misc_setSensitive(w_checkFile         , show_checkFile         , (gv_filename != NULL));
     misc_setSensitive(w_updateFile        , show_updateFile        , (gv_filename != NULL));
-    misc_setSensitive(w_showThisPage      , show_showThisPage      , (gv_psfile   != NULL));
+    misc_setSensitive(w_showThisPage      , show_showThisPage      , (gv_psfile   != NULL || (gv_gs_arguments && *gv_gs_arguments)));
     misc_setSensitive(w_prevPage          , show_prevPage          , (toc_text    != NULL));
     misc_setSensitive(w_nextPage          , show_nextPage          , (gv_filename != NULL));
     misc_setSensitive(w_toggleCurrentPage , show_toggleCurrentPage , (toc_text    != NULL));
@@ -1074,12 +1080,13 @@ setup_ghostview()
 
     XtSetSensitive(reopenEntry,      (gv_psfile   != NULL));
     XtSetSensitive(saveposEntry,      (gv_psfile   != NULL));
+    XtSetSensitive(presentationEntry,      (gv_psfile   != NULL));
     XtSetSensitive(printAllEntry,    (gv_psfile   != NULL));
     XtSetSensitive(printMarkedEntry, (toc_text    != NULL));
     XtSetSensitive(saveAllEntry,     (gv_psfile   != NULL));
     XtSetSensitive(saveMarkedEntry,  (toc_text    != NULL));
     XtSetSensitive(nextEntry,        (gv_filename != NULL));
-    XtSetSensitive(redisplayEntry,   (gv_psfile   != NULL));
+    XtSetSensitive(redisplayEntry,   (gv_psfile   != NULL || (gv_gs_arguments && *gv_gs_arguments)));
     XtSetSensitive(prevEntry,        (toc_text    != NULL));
     XtSetSensitive(currentEntry,     (toc_text    != NULL));
     XtSetSensitive(oddEntry,         (toc_text    != NULL));
@@ -1251,6 +1258,20 @@ set_new_scale(void)
        
        ascale = (float)viewClip->core.width / dx / 72.0 * default_xdpi;
     }
+    else if (fabs(ascale+3) <= 0.001)
+    {
+       int dx = current_urx - current_llx + 1;
+       int dy = current_ury - current_lly + 1;
+       
+       if (gv_orientation == 2 || gv_orientation == 3)
+       {
+          int hlp = dx;
+         dx = dy;
+         dy = hlp;
+       }
+       
+       ascale = (float)viewClip->core.height / dy / 72.0 * default_xdpi;
+    }
     else if (fabs(ascale+2) <= 0.001)
     {
        ascale = gv_ascale;
