@@ -40,8 +40,8 @@
 #define  VERSION_APPLIC_CONTEXT	app_con  			/* the Application Context */
 
 char *versionIdentification[] = {
-        "gv 3.6.6.91",
-        "February 2009",
+        "gv 3.6.7.90",
+        "December 2009",
 	0
 };
 
@@ -50,8 +50,8 @@ char *versionIdentification[] = {
    on HP/UX 9.07
    [suggested by Christian Illinger (illinger@lepsi.in2p3.fr)]
 */
-char versionResource[] = "gv 3.6.5.91";
-char versionCompatibility[] = "gv 3.6.5.91";
+char versionResource[] = "gv 3.6.7.90";
+char versionCompatibility[] = "gv 3.6.7.90";
                                             
 char * copyright = "\
  gv -- An X11 user interface for Ghostscript. \n\
